@@ -1,32 +1,5 @@
-///////////////////////////////////////////////////////////////////////////
-//
-// Copyright (c) 2012-2019 DreamWorks Animation LLC
-//
-// All rights reserved. This software is distributed under the
-// Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
-//
-// Redistributions of source code must retain the above copyright
-// and license notice and the following restrictions and disclaimer.
-//
-// *     Neither the name of DreamWorks Animation nor the names of
-// its contributors may be used to endorse or promote products derived
-// from this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-// IN NO EVENT SHALL THE COPYRIGHT HOLDERS' AND CONTRIBUTORS' AGGREGATE
-// LIABILITY FOR ALL CLAIMS REGARDLESS OF THEIR BASIS EXCEED US$250.00.
-//
-///////////////////////////////////////////////////////////////////////////
+// Copyright Contributors to the OpenVDB Project
+// SPDX-License-Identifier: MPL-2.0
 //
 /// @file Mat.h
 /// @author Joshua Schpok
@@ -139,6 +112,16 @@ public:
         return ostr;
     }
 
+    /// Direct access to the internal data
+    T* asPointer() { return mm; }
+    const T* asPointer() const { return mm; }
+
+    //@{
+    /// Array style reference to ith row
+    T* operator[](int i) { return &(mm[i*SIZE]); }
+    const T* operator[](int i) const { return &(mm[i*SIZE]); }
+    //@}
+
     void write(std::ostream& os) const {
         os.write(reinterpret_cast<const char*>(&mm), sizeof(T)*SIZE*SIZE);
     }
@@ -159,7 +142,7 @@ public:
     /// True if a Nan is present in this matrix
     bool isNan() const {
         for (unsigned i = 0; i < numElements(); ++i) {
-            if (std::isnan(mm[i])) return true;
+            if (math::isNan(mm[i])) return true;
         }
         return false;
     }
@@ -167,7 +150,7 @@ public:
     /// True if an Inf is present in this matrix
     bool isInfinite() const {
         for (unsigned i = 0; i < numElements(); ++i) {
-            if (std::isinf(mm[i])) return true;
+            if (math::isInfinite(mm[i])) return true;
         }
         return false;
     }
@@ -175,7 +158,7 @@ public:
     /// True if no Nan or Inf values are present
     bool isFinite() const {
         for (unsigned i = 0; i < numElements(); ++i) {
-            if (!std::isfinite(mm[i])) return false;
+            if (!math::isFinite(mm[i])) return false;
         }
         return true;
     }
@@ -958,7 +941,7 @@ lInfinityNorm(const MatType& matrix)
         typename MatType::ValueType column_sum = 0;
 
         for (int i = 0; i<n; ++i) {
-            column_sum += fabs(matrix(i,j));
+            column_sum += std::fabs(matrix(i,j));
         }
         norm = std::max(norm, column_sum);
     }
@@ -979,7 +962,7 @@ lOneNorm(const MatType& matrix)
         typename MatType::ValueType row_sum = 0;
 
         for (int j = 0; j<n; ++j) {
-            row_sum += fabs(matrix(i,j));
+            row_sum += std::fabs(matrix(i,j));
         }
         norm = std::max(norm, row_sum);
     }
@@ -1039,12 +1022,38 @@ polarDecomposition(const MatType& input, MatType& unitary,
     return true;
 }
 
+////////////////////////////////////////
+
+/// @return true if m0 < m1, comparing components in order of significance.
+template<unsigned SIZE, typename T>
+inline bool
+cwiseLessThan(const Mat<SIZE, T>& m0, const Mat<SIZE, T>& m1)
+{
+    const T* m0p = m0.asPointer();
+    const T* m1p = m1.asPointer();
+    constexpr unsigned size = SIZE*SIZE;
+    for (unsigned i = 0; i < size-1; ++i, ++m0p, ++m1p) {
+        if (!math::isExactlyEqual(*m0p, *m1p)) return *m0p < *m1p;
+    }
+    return *m0p < *m1p;
+}
+
+/// @return true if m0 > m1, comparing components in order of significance.
+template<unsigned SIZE, typename T>
+inline bool
+cwiseGreaterThan(const Mat<SIZE, T>& m0, const Mat<SIZE, T>& m1)
+{
+    const T* m0p = m0.asPointer();
+    const T* m1p = m1.asPointer();
+    constexpr unsigned size = SIZE*SIZE;
+    for (unsigned i = 0; i < size-1; ++i, ++m0p, ++m1p) {
+        if (!math::isExactlyEqual(*m0p, *m1p)) return *m0p > *m1p;
+    }
+    return *m0p > *m1p;
+}
+
 } // namespace math
 } // namespace OPENVDB_VERSION_NAME
 } // namespace openvdb
 
 #endif // OPENVDB_MATH_MAT_HAS_BEEN_INCLUDED
-
-// Copyright (c) 2012-2019 DreamWorks Animation LLC
-// All rights reserved. This software is distributed under the
-// Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
